//
//  GLMapDataSource.h
//  GLMapKit
//
//  Created by Jeffrey Early on 10/7/06.
//  Copyright 2006 Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

extern NSString *URLKey;
extern NSString *ImagePathKey;
extern NSString *ImageKey;
extern NSString *ZoneKey;
extern NSString *DSYCoordinateTopKey;
extern NSString *DSYCoordinateBottomKey;
extern NSString *DSXCoordinateLeftKey;
extern NSString *DSXCoordinateRightKey;
extern NSString *WidthKey;
extern NSString *HeightKey;
extern NSString *ScaleKey;
extern NSString *UniqueIdentifierKey;

@class GLMapView;
@interface GLMapDataSource : NSObject <NSURLDownloadDelegate>
{
	NSMutableDictionary				*mapTiles;
	
	NSMutableDictionary				*tileCacheDictionary;
	NSMutableDictionary				*urlUIDDictionary;
	NSMutableDictionary				*tilesQueuedDictionary;
	
	NSString						*defaultStoreString;
	NSString						*indexFilename;
	NSString						*name;
	NSMutableArray					*currentDownloads;
	NSMutableArray					*currentNSURLDownloads; // use to keep the objects from being GC'ed
	
	NSMutableArray					*TileDictionaryArray;
		
	NSMutableDictionary				*tilesCurrentlyDownloading;
	
	NSMapTable						*mapViewUIDTable;
	NSMapTable						*prefetchDelegateUIDTable;
	NSMapTable						*prefetchDelegateSelectorTable;
	
	NSLock							*tileStateChangeLock;
	
	BOOL downloadInProgress;
	int expectedNumberOfDownloads;
	int currentDownload;
}

/************************************************/
/*		Scale and Coordinate Properties			*/
/************************************************/

#pragma mark -
#pragma mark Scale and Coordinate Properties
#pragma mark

// Every unique datasource is required to provided its own 16-bit unique UID
//+ (uint16) uid;

- (uint16) uid;

// A subclass is required to override all of these methods, defining the datasource's basic capabilities

// user readable name of this data source
@property(readonly, copy) NSString *name;

// Override one or the other to display the license for the map tiles.
@property(readonly, copy) NSString *license;
@property(readonly, copy) NSAttributedString *attributedlicense;

@property(readonly, copy) NSString *tileFolderName;

// The data sources need to return the projection object that should be used at the given
// coordinates. The idea here is that a data source using the UTM system could return a different
// projection object depending on the zone.
- (GLMapProjection *) projectionForLongitudeLatitude: (GLPoint) latLong;

// Given some desired dsScale, what's the best map tile scale to use for screen display
- (double) bestResolutionForResolution: (double) resolution atCoordinates: (GLPoint) dsCoordinates withProjection: (GLMapProjection *) mapProjection;

// Given some desired dsScale, what's the best map tile scale to use for printing
- (double) bestPrintResolutionForResolution: (double) resolution atCoordinates: (GLPoint) dsCoordinates withProjection: (GLMapProjection *) mapProjection;


/************************************************/
/*		Tile Fetching Subclass Overrides		*/
/************************************************/

#pragma mark -
#pragma mark Tile Fetching Subclass Overrides
#pragma mark

// Almost all of the major work goes on with these three functions. Of course, the resolution and coordinates properties
// also have to be overridden by a subclass, but these define which tiles to fetch for which areas.

// Returns an array of UID strings
- (NSArray *) uidsForRect: (GLRect) coordinates atResolution: (double) resolution withProjection: (GLMapProjection *) mapProjection;

// returns the http url for a given UID (if applicable)
- (NSString *) imageURLForUID: (NSString *) aUID;

// returns a dictionary containing, at minimum, the following keys:
// ImagePathKey, WidthKey, HeightKey, ScaleKey, UniqueIdentifierKey
// DSYCoordinateTopKey, DSYCoordinateBottomKey, DSXCoordinateLeftKey, DSXCoordinateRightKey.
- (NSDictionary *) tileForUID: (NSString *) aUID;


/************************************************/
/*		Tile Fetching							*/
/************************************************/

#pragma mark -
#pragma mark Tile Fetching
#pragma mark

// Called by GLMapView. Request is in DSCoordinates. Tiles can be returned lazily
// by calling [mapView addMapTiles: (NSArray *) array] where the array is an array
// of dictionaries.
- (void) tilesForRect: (GLRect) coordinates atResolution: (double) resolution withProjection: (GLMapProjection *) mapProjection forMapView: (GLMapView *) mapView;

// Called by GLMapView. Request is in DSCoordinates. All available tiles must be returned now.
// This method does NOT start the download of missing tiles.
- (NSArray *) tilesNowForRect: (GLRect) coordinates atResolution: (double) resolution withProjection: (GLMapProjection *) mapProjection;

// Called by GLMapView. Request is in DSCoordinates.
- (void) prefetchTilesForRect: (GLRect) coordinates atResolution: (double) resolution withProjection: (GLMapProjection *) mapProjection;

// Same as above, but sets a selector to be called when the prefetch is complete
- (void) prefetchTilesForRect: (GLRect) coordinates atResolution: (double) resolution withProjection: (GLMapProjection *) mapProjection delegate: (id) aDelegate didFinishSelector: (SEL) selector;

// These properties are to help with a download dialog
@property(nonatomic) BOOL downloadInProgress;
@property int expectedNumberOfDownloads;
@property int currentDownload;


/************************************************/
/*		Private Methods							*/
/************************************************/

#pragma mark -
#pragma mark Private Methods
#pragma mark

// Update and reset (after new request) download statitics
- (void) resetDownloadStats;
- (void) incrementDownload;

// returns a dictionary containing tiles currently being downloaded (key, value) = (uid, url)
@property(copy) NSMutableDictionary *tilesCurrentlyDownloading;

// MapView objects as keys, whos value is an array containing the last requested
// UIDs that were not able to be sent at the time.
@property(readonly) NSMapTable *mapViewUIDTable;

// Keys contains the prefetch delegate in both case. The first table's objects are arrays of UIDs that are supposed to be prefetched.
// The second table's objects are invocations to be invoke once the prefetch is complete. In both cases the keys are weak, pointers only,
// so that if the delegate vanishes they will be removed from the prefetch tables.
@property(readonly) NSMapTable *prefetchDelegateUIDTable;
@property(readonly) NSMapTable *prefetchDelegateSelectorTable;

// returns nil if the tile isn't cached.
- (NSString *) tileCachePathForUID: (NSString *) aUID;

// returns yes if the tile is currently being downloaded (or in the queue).
- (BOOL) tileCurrentlyDownloadingWithUID:  (NSString *) aUID;

// downloads and then attempts to return the download to mapView
- (void) downloadURL: (NSString *) url forUID: (NSString *) aUID;

// Private method for initializing the map tile cache
- (void) setupMapTileCache;

@property(readwrite, strong) NSMutableDictionary *tileCacheDictionary;

@end
